import 'package:proto_annotations/proto_annotations.dart';
import 'package:proto_generator/src/proto_mapper/standalone_field_code_generator.dart';
import 'package:recase/recase.dart';

class IntFieldCodeGenerator extends StandaloneFieldCodeGenerator {
  IntFieldCodeGenerator({
    required super.fieldDescriptor,
    required super.refName,
    required super.protoRefName,
  });

  bool get _isInt64 {
    return fieldDescriptor.protoField.intPrecision == IntPrecision.int64;
  }

  @override
  String get toProtoMap {
    if (_isInt64) {
      return toProtoMap64;
    }
    return super.toProtoMap;
  }

  String get toProtoMap64 => fieldDescriptor.isNullable
      ? '''
        if ($ref$fieldName != null) {
          $protoRef$protoFieldName = Int64($ref$fieldName!); 
        }
      '''
      : '$protoRef$protoFieldName = Int64($ref$fieldName);';

  @override
  String get fromProtoMap {
    if (_isInt64) {
      return fromProtoMap64;
    }
    return super.fromProtoMap;
  }

  String get fromProtoMap64 {
    if (fieldDescriptor.isNullable) {
      return '''(${protoRef}has${protoFieldName.pascalCase}() ? $protoRef$protoFieldName.toInt() : null)''';
    }
    return '$protoRef$protoFieldName.toInt()';
  }
}
