import 'package:proto_annotations/proto_annotations.dart';
import 'package:proto_generator/src/proto_mapper/wkt_field_code_generator.dart';
import 'package:squarealfa_generators_common/squarealfa_generators_common.dart';

class GIntFieldCodeGenerator extends WKTFieldCodeGenerator {
  GIntFieldCodeGenerator({
    required super.fieldDescriptor,
    required super.refName,
    required super.protoRefName,
  });

  bool get _isInt64 {
    return fieldDescriptor.protoField.intPrecision == IntPrecision.int64;
  }

  @override
  String get toProtoMap {
    if (_isInt64) {
      return toInt64ProtoMap;
    }
    return toInt32ProtoMap;
  }

  String get toInt32ProtoMap {
    return fieldDescriptor.isNullable
        ? '''
        if ($ref$fieldName != null) {
          $protoRef$protoFieldName = Int32Value() .. value= $ref$fieldName!; 
        }
      '''
        : '$protoRef$protoFieldName = $ref$fieldName;';
  }

  String get toInt64ProtoMap {
    return fieldDescriptor.isNullable
        ? '''
        if ($ref$fieldName != null) {
          $protoRef$protoFieldName = Int64Value() .. value= Int64($ref$fieldName!); 
        }
      '''
        : '$protoRef$protoFieldName = Int64($ref$fieldName);';
  }

  @override
  String get fromProtoMap {
    if (_isInt64) {
      return fromInt64ProtoMap;
    }
    return fromInt32ProtoMap;
  }

  String get fromInt32ProtoMap {
    if (fieldDescriptor.isNullable) {
      return '''(${protoRef}has${protoFieldName.pascalName}() ? $protoRef$protoFieldName.value : null)''';
    }
    return '$protoRef$protoFieldName';
  }
  String get fromInt64ProtoMap {
    if (fieldDescriptor.isNullable) {
      return '''(${protoRef}has${protoFieldName.pascalName}() ? $protoRef$protoFieldName.value.toInt() : null)''';
    }
    return '$protoRef$protoFieldName.toInt()';
  }
}
